
#ifndef _LCDH_
#define _LCDH_

#if ARDUINO >= 100
 #include "Arduino.h"
 #include "Print.h"
#else
 #include "WProgram.h"
#endif
#include "ERGFX.h"
#ifdef __AVR__
 #include <avr/pgmspace.h>
#endif

////Landscape screen
#define LCD_TFTWIDTH  800   
#define LCD_TFTHEIGHT 480
// Vertical screen
//#define LCD_TFTWIDTH  480  
//#define LCD_TFTHEIGHT 800



// Color definitions
#define	LCD_BLACK   0x0000
#define	LCD_BLUE    0x001F
#define	LCD_RED     0xF800
#define	LCD_GREEN   0x07E0
#define LCD_CYAN    0x07FF
#define LCD_MAGENTA 0xF81F
#define LCD_YELLOW  0xFFE0  
#define LCD_WHITE   0xFFFF


class LCD : public ERGFX {

 public:

  LCD(int8_t _CS,int8_t _DC,int8_t _RST,int8_t _WR,int8_t _RD,int8_t _D0,int8_t _D1,int8_t _D2,int8_t _D3,int8_t _D4,int8_t _D5,int8_t _D6,int8_t _D7);
 

  void     begin(void),
           setAddrWindow(uint16_t x0, uint16_t y0, uint16_t x1, uint16_t y1),
           pushColor(uint16_t color),
           fillScreen(uint16_t color),
           drawPixel(int16_t x, int16_t y, uint16_t color),
           drawFastVLine(int16_t x, int16_t y, int16_t h, uint16_t color),
           drawFastHLine(int16_t x, int16_t y, int16_t w, uint16_t color),
           fillRect(int16_t x, int16_t y, int16_t w, int16_t h,
             uint16_t color),
           bmp16(uint16_t x, uint16_t y, const uint8_t *pBmp, uint16_t chWidth, uint16_t chHeight); 
  uint16_t color565(uint8_t r, uint8_t g, uint8_t b);

  /* These are not for current use, 8-bit protocol only! */
  uint16_t  readdata(void),
    readcommand8(uint16_t reg, uint16_t index = 0);
  /*
  uint16_t readcommand16(uint8_t);
  uint32_t readcommand32(uint8_t);
  void     dummyclock(void);
  */  

  void     writebyte(uint8_t),
    writecommand(uint16_t c),
    writedata(uint8_t d),
    writecommandword(uint8_t d),
    commandList(uint16_t *addr);
 

 private:
  uint8_t  tabcolor;




#if defined (__AVR__) || defined(TEENSYDUINO)
  uint8_t mySPCR;
  volatile uint8_t  *wrport, *rdport, *dcport, *rsport, *csport, *d0port, *d1port, *d2port, *d3port, *d4port, *d5port, *d6port, *d7port;
  int8_t    _cs, _dc, _rst,  _wr,  _rd,  _d0,  _d1,  _d2,  _d3,  _d4,  _d5,  _d6,  _d7;
  uint8_t  cspinmask, dcpinmask,   wrpinmask,  rdpinmask,  d0pinmask,  d1pinmask,  d2pinmask,  d3pinmask,  d4pinmask,  d5pinmask,  d6pinmask,  d7pinmask;
#elif defined (__arm__)
    volatile RwReg  *wrport, *rdport, *dcport, *rsport, *csport, *d0port, *d1port, *d2port, *d3port, *d4port, *d5port, *d6port, *d7port;
    uint32_t   _cs, _dc, _rst,  _wr,  _rd,  _d0,  _d1,  _d2,  _d3,  _d4,  _d5,  _d6,  _d7;
    uint32_t  cspinmask, dcpinmask,   wrpinmask,  rdpinmask,  d0pinmask,  d1pinmask,  d2pinmask,  d3pinmask,  d4pinmask,  d5pinmask,  d6pinmask,  d7pinmask;

  


#elif defined(ESP32)

   volatile uint32_t  *wrport, *rdport, *dcport, *rsport, *csport, *d0port, *d1port, *d2port, *d3port, *d4port, *d5port, *d6port, *d7port;
    uint32_t  _cs, _dc, _rst,  _wr,  _rd,  _d0,  _d1,  _d2,  _d3,  _d4,  _d5,  _d6,  _d7;

    uint32_t  cspinmask, dcpinmask,   wrpinmask,  rdpinmask,  d0pinmask,  d1pinmask,  d2pinmask,  d3pinmask,  d4pinmask,  d5pinmask,  d6pinmask,  d7pinmask;
#endif       

};

#endif
